"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var JsonObjectConfigurationProvider = /** @class */ (function () {
    function JsonObjectConfigurationProvider(configObject) {
        this.configObject = configObject;
        this.configObject = configObject || {};
    }
    JsonObjectConfigurationProvider.prototype.getAllKeyValues = function (callback) {
        return callback(null, this.configObject);
    };
    JsonObjectConfigurationProvider.prototype.getName = function () {
        return "JSON Object";
    };
    return JsonObjectConfigurationProvider;
}());
exports.JsonObjectConfigurationProvider = JsonObjectConfigurationProvider;
var EnvVariableConfigurationProvider = /** @class */ (function () {
    function EnvVariableConfigurationProvider(prefix) {
        this.prefix = prefix;
    }
    EnvVariableConfigurationProvider.prototype.getAllKeyValues = function (callback) {
        if (this.prefix) {
            var ret = {};
            for (var key in process.env) {
                if (key.indexOf(this.prefix) == 0) {
                    ret[key.substr(this.prefix.length)] = process.env[key];
                }
            }
            callback(null, ret);
        }
        else {
            return callback(null, process.env);
        }
    };
    EnvVariableConfigurationProvider.prototype.getName = function () {
        return "Environment Variables";
    };
    return EnvVariableConfigurationProvider;
}());
exports.EnvVariableConfigurationProvider = EnvVariableConfigurationProvider;
var ConfigProviderAggregator = /** @class */ (function () {
    function ConfigProviderAggregator(providers) {
        this.providers = providers;
        if (!providers)
            throw new Error("no provider was specified");
    }
    ConfigProviderAggregator.prototype.getAllKeyValues = function (callback) {
        var flattenedConfiguration = {};
        var remainingProviders = [].concat(this.providers);
        var attemptNext = function () {
            if (remainingProviders.length == 0) {
                return callback(null, flattenedConfiguration);
            }
            var nextProvider = remainingProviders.shift();
            nextProvider.getAllKeyValues(function (err, keysAndValues) {
                if (err)
                    return attemptNext();
                Object.keys(keysAndValues).forEach(function (k) {
                    if (!flattenedConfiguration.hasOwnProperty(k))
                        flattenedConfiguration[k] = keysAndValues[k];
                });
                return attemptNext();
            });
        };
        attemptNext();
    };
    ConfigProviderAggregator.prototype.getName = function () {
        return "Aggregated config from multiple providers: " + this.providers.map(function (t) { return t.getName(); });
    };
    return ConfigProviderAggregator;
}());
exports.ConfigProviderAggregator = ConfigProviderAggregator;
var JsonConfigFileConfigurationProvider = /** @class */ (function () {
    function JsonConfigFileConfigurationProvider(filename) {
        this.filename = filename;
        if (!filename)
            throw new Error("filename is required");
    }
    JsonConfigFileConfigurationProvider.prototype.getAllKeyValues = function (callback) {
        try {
            var cfg = JSON.parse(fs.readFileSync(this.filename).toString().trim());
            return callback(null, cfg);
        }
        catch (e) {
            return callback(e, null);
        }
    };
    JsonConfigFileConfigurationProvider.prototype.getName = function () {
        return "Config filename: " + this.filename;
    };
    return JsonConfigFileConfigurationProvider;
}());
exports.JsonConfigFileConfigurationProvider = JsonConfigFileConfigurationProvider;
var AgentConfigKey = /** @class */ (function () {
    function AgentConfigKey() {
    }
    return AgentConfigKey;
}());
exports.AgentConfigKey = AgentConfigKey;
var StringConfigKey = /** @class */ (function () {
    function StringConfigKey(required, defaultValue) {
        this.required = required;
        this.isConfigKey = true;
        this.metadata = { required: required, type: "string" };
        if (defaultValue !== undefined) {
            this.value = defaultValue;
        }
    }
    Object.defineProperty(StringConfigKey.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            this._value = value;
            this.hasValue = true;
        },
        enumerable: true,
        configurable: true
    });
    StringConfigKey.prototype.loadFromRawData = function (s) {
        this.value = s;
        this.hasValue = true;
    };
    return StringConfigKey;
}());
exports.StringConfigKey = StringConfigKey;
var NumberConfigKey = /** @class */ (function () {
    function NumberConfigKey(required, defaultValue) {
        this.required = required;
        this.isConfigKey = true;
        this.metadata = { required: required, type: "number" };
        if (defaultValue !== undefined) {
            this.value = defaultValue;
        }
    }
    Object.defineProperty(NumberConfigKey.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            this._value = value;
            this.hasValue = true;
        },
        enumerable: true,
        configurable: true
    });
    NumberConfigKey.prototype.loadFromRawData = function (s) {
        var parsed = parseInt(s);
        if (isNaN(parsed))
            throw new Error(s + " is not a valid number");
        this.value = parsed;
        this.hasValue = true;
    };
    return NumberConfigKey;
}());
exports.NumberConfigKey = NumberConfigKey;
var BooleanConfigKey = /** @class */ (function () {
    function BooleanConfigKey(required, defaultValue) {
        this.required = required;
        this.isConfigKey = true;
        this.metadata = { required: required, type: "boolean" };
        if (defaultValue !== undefined) {
            this.value = defaultValue;
        }
    }
    Object.defineProperty(BooleanConfigKey.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            this._value = value;
            this.hasValue = true;
        },
        enumerable: true,
        configurable: true
    });
    BooleanConfigKey.prototype.loadFromRawData = function (s) {
        if (s == null)
            s = "";
        if (typeof s === "boolean")
            s = String(s);
        s = s.toLowerCase();
        switch (s) {
            case "true":
                this.value = true;
                this.hasValue = true;
                return;
            case "false":
                this.value = false;
                this.hasValue = true;
                return;
            case "undefined":
                this.value = false;
                this.hasValue = false;
                return;
            default:
                throw new Error("Invalid boolean value: " + s);
        }
    };
    return BooleanConfigKey;
}());
exports.BooleanConfigKey = BooleanConfigKey;
var BaseConfiguration = /** @class */ (function () {
    function BaseConfiguration() {
    }
    BaseConfiguration.prototype.loadConfigurationFromMultipleProviders = function (configProviders, callback) {
        var provider = new ConfigProviderAggregator(configProviders);
        this.loadConfiguration(provider, callback);
    };
    BaseConfiguration.prototype.loadConfiguration = function (configProvider, callback, dbg) {
        var _this = this;
        try {
            configProvider.getAllKeyValues(function (err, keysAndValues) {
                if (err) {
                    if (callback) {
                        callback(err);
                    }
                    return;
                }
                keysAndValues = keysAndValues || {};
                var knownKeys = Object.keys(_this);
                var hadError = false;
                var keyWithError = "";
                knownKeys.forEach(function (keyName) {
                    if (hadError) {
                        console.log("[Sealights Test Listener] - Failed to load configuration due to invalid value in '" + keyWithError + "' field'.");
                        return;
                    }
                    var cfgKey = _this[keyName];
                    if (!cfgKey.isConfigKey)
                        return;
                    var rawValue = keysAndValues[keyName];
                    if (cfgKey.metadata.required && rawValue == undefined) {
                        var msg = "Required configuration is missing: " + keyName + ", provider:" + configProvider.getName();
                        hadError = true;
                        if (callback) {
                            callback(new Error(msg));
                        }
                    }
                    if (rawValue == undefined)
                        return; //no value specified and cfg is not required
                    try {
                        cfgKey.loadFromRawData(rawValue);
                    }
                    catch (e) {
                        var msg = "Invalid configuration for key=" + keyName + ", value=" + rawValue + +", provider=" + configProvider.getName() + ": " + e.toString();
                        console.log("[Sealights Test Listener] - " + msg);
                        hadError = true;
                        keyWithError = keyName;
                        if (callback) {
                            callback(new Error(msg));
                        }
                    }
                });
                if (!hadError) {
                    if (callback) {
                        callback(null);
                    }
                    return;
                }
            });
        }
        catch (e) {
            if (callback) {
                callback(e);
            }
            return;
        }
    };
    BaseConfiguration.prototype.toJsonObject = function () {
        var _this = this;
        var ret = {};
        var knownKeys = Object.keys(this);
        knownKeys.forEach(function (keyName) {
            var cfgKey = _this[keyName];
            if (cfgKey.isConfigKey && cfgKey.hasValue) {
                ret[keyName] = cfgKey.value;
            }
        });
        return ret;
    };
    BaseConfiguration.prototype.getLowerCaseToKeyMap = function () {
        var lowerCaseMap = {};
        Object.keys(this).forEach(function (key) {
            lowerCaseMap[key.toLowerCase()] = key;
        });
        return lowerCaseMap;
    };
    return BaseConfiguration;
}());
exports.BaseConfiguration = BaseConfiguration;
//# sourceMappingURL=config-system.js.map