"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var validation_utils_1 = require("../utils/validation-utils");
var files_utils_1 = require("../utils/files-utils");
var source_maps_utils_1 = require("../source-maps-utils");
/*
    This class iterate over all elements in istanbul coverage data and checks if they are generated from other files.
    if so we read source maps data and load the original module into istanbul modules.
    use cases:
        1. retrieve '.ts' files from generated '.js' files.
        2. retrieve '.js' files from 'bundle.js' that generated using webpack.
 */
var BRANCHES_MAP_KEY = "branchMap";
var METHODS_MAP_KEY = "fnMap";
var OriginalModuleLoader = /** @class */ (function () {
    function OriginalModuleLoader(coverageObject, logger) {
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(coverageObject, "coverageObject");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(logger, "logger");
        this.coverageObject = coverageObject;
        this.logger = logger;
        this._fileToSourceMapConsumer = {};
    }
    OriginalModuleLoader.prototype.load = function () {
        var _this = this;
        var modules = Object.keys(this.coverageObject);
        modules.map(function (moduleName) { return _this.tryLoadOriginalModule(moduleName); });
    };
    OriginalModuleLoader.prototype.tryLoadOriginalModule = function (istanbulModule) {
        if (!this.getSourceMapForFile(istanbulModule)) {
            // No source maps for file, nothing to load.
            return;
        }
        this.readOriginalModule(istanbulModule);
    };
    OriginalModuleLoader.prototype.readOriginalModule = function (moduleName) {
        var _this = this;
        var sourceMapsReader = this.getSourceMapForFile(moduleName);
        var module = this.coverageObject[moduleName];
        Object.keys(module.branchMap).forEach(function (key) { return _this.resolveAndAddBranch(module.branchMap[key], sourceMapsReader, moduleName); });
        Object.keys(module.fnMap).map(function (key) { return _this.resolveAndAddMethod(module.fnMap[key], sourceMapsReader, moduleName); });
    };
    OriginalModuleLoader.prototype.resolveAndAddMethod = function (method, sourceMapsReader, moduleName) {
        var originalMethod = this.createEmptyMethodObject();
        originalMethod.name = method.name;
        var originalModuleName = this.loadOriginalDataForElement(sourceMapsReader, method, moduleName, originalMethod);
        if (originalModuleName) {
            this.addElementToModule(originalModuleName, originalMethod, METHODS_MAP_KEY);
        }
    };
    OriginalModuleLoader.prototype.resolveAndAddBranch = function (branch, sourceMapsReader, moduleName) {
        var _this = this;
        var originalBranch = this.createEmptyBranchObject();
        originalBranch.type = branch.type;
        var originalModuleName = this.loadOriginalDataForElement(sourceMapsReader, branch, moduleName, originalBranch);
        if (originalModuleName) {
            var originalLocations = branch.locations.map(function (locationElement) { return _this.createElementSourceData(locationElement, sourceMapsReader, moduleName); });
            originalBranch.locations = originalLocations;
            this.addElementToModule(originalModuleName, originalBranch, BRANCHES_MAP_KEY);
        }
    };
    OriginalModuleLoader.prototype.loadOriginalDataForElement = function (sourceMapsReader, element, moduleName, originalElement) {
        var sourceMapDataForStart = this.readSourceMapData(sourceMapsReader, element.loc.start, moduleName);
        var sourceMapDataForEnd = this.readSourceMapData(sourceMapsReader, element.loc.end, moduleName);
        if (!sourceMapDataForStart) {
            // current method has no source data
            return;
        }
        ;
        originalElement.loc.start = sourceMapDataForStart.originalPosition;
        originalElement.line = sourceMapDataForStart.originalPosition.line;
        var originalModuleName = sourceMapDataForStart.originalFilename;
        if (sourceMapDataForEnd) {
            originalElement.loc.end = sourceMapDataForEnd.originalPosition;
        }
        if (element.decl) {
            originalElement.decl = {};
            var sourceMapDataForDeclStart = this.readSourceMapData(sourceMapsReader, element.decl.start, moduleName);
            var sourceMapDataForDeclEnd = this.readSourceMapData(sourceMapsReader, element.decl.end, moduleName);
            if (sourceMapDataForDeclStart) {
                originalElement.decl.start = sourceMapDataForDeclStart.originalPosition;
            }
            if (sourceMapDataForDeclEnd) {
                originalElement.decl.end = sourceMapDataForDeclEnd.originalPosition;
            }
        }
        return originalModuleName;
    };
    OriginalModuleLoader.prototype.addElementToModule = function (moduleName, element, mapObjectKey) {
        var module = this.getOrCreateIstanbulModule(moduleName);
        var key = this.createElementKey(element);
        if (!module[mapObjectKey][key]) {
            module[mapObjectKey][key] = element;
        }
    };
    OriginalModuleLoader.prototype.createElementSourceData = function (locationElement, sourceMapsReader, moduleName) {
        var sourceMapDataForStart = this.readSourceMapData(sourceMapsReader, locationElement.start, moduleName);
        var sourceMapDataForEnd = this.readSourceMapData(sourceMapsReader, locationElement.end, moduleName);
        if (!sourceMapDataForStart && !sourceMapDataForEnd) {
            return {};
        }
        var sourceLocationElement = {
            start: sourceMapDataForStart.originalPosition,
            end: sourceMapDataForEnd.originalPosition,
        };
        return sourceLocationElement;
    };
    OriginalModuleLoader.prototype.getOrCreateIstanbulModule = function (moduleName) {
        var module = this.coverageObject[moduleName];
        if (!module) {
            module = this.createIstanbulModule(moduleName);
            this.coverageObject[moduleName] = module;
        }
        return module;
    };
    OriginalModuleLoader.prototype.getSourceMapForFile = function (fullPath) {
        if (!this._fileToSourceMapConsumer[fullPath]) {
            this._fileToSourceMapConsumer[fullPath] = source_maps_utils_1.SourceMapsUtils.readSourceMaps(fullPath);
            this.logger.debug("Read source maps for file':" + fullPath + "'");
        }
        var sourceMap = this._fileToSourceMapConsumer[fullPath];
        return sourceMap;
    };
    OriginalModuleLoader.prototype.readSourceMapData = function (sourceMaps, start, fullPath) {
        if (!sourceMaps)
            return null;
        try {
            var originalPosition = sourceMaps.originalPositionFor(start); //{ line: start.line, column: start[1] });
            if (originalPosition && originalPosition.source && originalPosition.line !== null && originalPosition.column !== null) {
                var originalFilename = originalPosition.source;
                if (originalFilename.indexOf("webpack:///webpack/") === -1 &&
                    originalFilename.indexOf("webpack:///external") === -1) { //Ignore embedded and external module
                    if (originalFilename.indexOf('webpack:///') === 0) {
                        originalFilename = originalFilename.substring(11); //'webpack:///'.length
                    }
                    originalFilename = files_utils_1.FilesUtils.resolveOriginalFullFileName(fullPath, originalFilename);
                    var originalPositionObj = { line: originalPosition.line, column: originalPosition.column };
                    var result = { originalFilename: originalFilename, originalPosition: originalPositionObj };
                    return result;
                }
            }
        }
        catch (e) {
            console.log(e);
        }
        return null;
    };
    OriginalModuleLoader.prototype.createElementKey = function (element) {
        return element.loc.start.line + ":" + element.loc.start.column;
    };
    OriginalModuleLoader.prototype.createEmptyBranchObject = function () {
        return {
            loc: {},
            locations: [],
            type: "",
            line: null
        };
    };
    OriginalModuleLoader.prototype.createEmptyMethodObject = function () {
        return {
            loc: {},
            name: "",
            line: null
        };
    };
    OriginalModuleLoader.prototype.createIstanbulModule = function (path) {
        return {
            s: {},
            f: {},
            b: {},
            fnMap: {},
            statementMap: {},
            branchMap: {},
            path: path,
            uniqueIdsMap: null
        };
    };
    Object.defineProperty(OriginalModuleLoader.prototype, "fileToSourceMapConsumer", {
        get: function () {
            return this._fileToSourceMapConsumer;
        },
        set: function (value) {
            this._fileToSourceMapConsumer = value;
        },
        enumerable: true,
        configurable: true
    });
    return OriginalModuleLoader;
}());
exports.OriginalModuleLoader = OriginalModuleLoader;
//# sourceMappingURL=original-module-loader.js.map