"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var contracts_1 = require("./contracts");
var file_element_1 = require("./file-element");
var METHOD_DELIMITER = "@";
var BRANCH_DELIMITER = "|";
var NULL_PARAM_MESSAGE = "cannot be null or undefined";
/*
    this class create uniqueId's in new format for build mapping input.
 */
var UniqueIdConverter = /** @class */ (function () {
    function UniqueIdConverter(filename, logger) {
        if (!filename) {
            throw new Error("'filename' " + NULL_PARAM_MESSAGE);
        }
        if (!logger) {
            throw new Error("'logger' " + NULL_PARAM_MESSAGE);
        }
        this._filename = filename;
        this.logger = logger;
        this._uniqueIdsMap = {};
        this._methodsArray = [];
        this._branchesArray = [];
    }
    UniqueIdConverter.prototype.sortElements = function () {
        this._methodsArray = this._methodsArray.sort(this.comparePosition);
        this._branchesArray = this._branchesArray.sort(this.comparePosition);
    };
    UniqueIdConverter.prototype.createElementsArray = function (file) {
        file.methods = file.methods || [];
        file.branches = file.branches || [];
        this.fillFileElementsArray(file.methods, this._methodsArray, contracts_1.ElementType.METHOD);
        this.fillFileElementsArray(file.branches, this._branchesArray, contracts_1.ElementType.BRANCH);
    };
    UniqueIdConverter.prototype.createFileElement = function (element, filename, type) {
        var startPosition = this.createElementPosition(element.position);
        var endPosition = this.createElementPosition(element.endPosition);
        var fileElement = new file_element_1.FileElement(type, startPosition, endPosition, element.uniqueId, filename);
        if (element.uniqueId_decl) {
            fileElement.uniqueIdKey_decl = element.uniqueId_decl;
        }
        /*
         Istanbul reports 'if' and 'else' in same position. old uniqueId is based on parent position (equal to the 'if'
         location). so here we consider the parent position as start position, but to keep branches order we add the
         'index' value to column.
         */
        if (element.parentPosition) {
            var index = element.index || 0;
            fileElement.startPosition = this.createElementPosition(element.parentPosition);
            fileElement.index = index;
        }
        return fileElement;
    };
    UniqueIdConverter.prototype.process = function () {
        this.removeDuplicateElements(this._methodsArray);
        this.removeDuplicateElements(this._branchesArray);
        this.sortElements();
        this.removeBranchesOutOfMethods();
        this.logger.debug("sorted " + this._branchesArray.length + " branches");
        this.logger.debug("sorted " + this._methodsArray.length + " methods");
        this.fillUniqueIdsMap();
    };
    UniqueIdConverter.prototype.setAndInitFile = function (file) {
        this.createElementsArray(file);
    };
    UniqueIdConverter.prototype.createElementPosition = function (position) {
        var line;
        var column;
        if (Array.isArray(position) && position.length === 2) {
            line = parseInt(position[0]);
            column = parseInt(position[1]);
        }
        else {
            throw new Error("element has no correct startPosition array, cannot resolve startPosition");
        }
        return {
            column: column,
            line: line,
        };
    };
    UniqueIdConverter.prototype.fillFileElementsArray = function (elements, array, type) {
        for (var _i = 0, elements_1 = elements; _i < elements_1.length; _i++) {
            var element = elements_1[_i];
            array.push(this.createFileElement(element, this._filename, type));
        }
    };
    UniqueIdConverter.prototype.fillUniqueIdsMap = function () {
        this.createNewUniqueIds(this._methodsArray, METHOD_DELIMITER);
        this.createNewUniqueIds(this._branchesArray, BRANCH_DELIMITER);
    };
    UniqueIdConverter.prototype.createNewUniqueIds = function (array, delimiter) {
        var aggregatedByLine = this.aggregateElementsByLine(array);
        var lines = Object.keys(aggregatedByLine);
        for (var _i = 0, lines_1 = lines; _i < lines_1.length; _i++) {
            var line = lines_1[_i];
            var indexesArray = aggregatedByLine[line];
            for (var i = 0; i < indexesArray.length; i++) {
                var currentIndex = indexesArray[i];
                var currentElement = array[currentIndex];
                currentElement.uniqueId = currentElement.filename + delimiter + line + "," + i;
                this.logger.debug("[UniqueIdConverter] uniqueId " + currentElement.uniqueIdKey + " converted to \n                               " + currentElement.uniqueId);
                this._uniqueIdsMap[currentElement.uniqueIdKey] = currentElement.uniqueId;
                if (currentElement.uniqueIdKey_decl) {
                    this._uniqueIdsMap[currentElement.uniqueIdKey_decl] = currentElement.uniqueId;
                }
            }
        }
    };
    UniqueIdConverter.prototype.comparePosition = function (elm1, elm2) {
        if (elm1.startPosition.line < elm2.startPosition.line) {
            return -1;
        }
        if (elm1.startPosition.line > elm2.startPosition.line) {
            return 1;
        }
        if (elm1.startPosition.column < elm2.startPosition.column) {
            return -1;
        }
        if (elm1.startPosition.column > elm2.startPosition.column) {
            return 1;
        }
        if (elm1.index < elm2.index) {
            return -1;
        }
        if (elm1.index > elm2.index) {
            return 1;
        }
        return 0;
    };
    Object.defineProperty(UniqueIdConverter.prototype, "uniqueIdsMap", {
        get: function () {
            return this._uniqueIdsMap;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniqueIdConverter.prototype, "methodsArray", {
        get: function () {
            return this._methodsArray;
        },
        set: function (value) {
            this._methodsArray = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniqueIdConverter.prototype, "branchesArray", {
        get: function () {
            return this._branchesArray;
        },
        set: function (value) {
            this._branchesArray = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniqueIdConverter.prototype, "filename", {
        get: function () {
            return this._filename;
        },
        enumerable: true,
        configurable: true
    });
    UniqueIdConverter.prototype.removeDuplicateElements = function (elementsArray) {
        var _this = this;
        var existedKeys = {};
        elementsArray = elementsArray.filter(function (element) {
            return _this.filterDuplicates(element, existedKeys);
        });
    };
    UniqueIdConverter.prototype.filterDuplicates = function (element, existedKeys) {
        if (existedKeys[element.uniqueIdKey] ||
            (element.uniqueIdKey_decl && existedKeys[element.uniqueIdKey_decl])) {
            return false;
        }
        existedKeys[element.uniqueIdKey] = true;
        if (element.uniqueIdKey_decl) {
            existedKeys[element.uniqueIdKey_decl] = true;
        }
        return true;
    };
    ;
    UniqueIdConverter.prototype.removeBranchesOutOfMethods = function () {
        var validBranches = [];
        var methodsIndex = 0;
        for (var _i = 0, _a = this._branchesArray; _i < _a.length; _i++) {
            var branch = _a[_i];
            for (var i = methodsIndex; i < this._methodsArray.length; i++) {
                var currentMethod = this._methodsArray[i];
                var branchPosition = this.checkBranchPosition(branch, currentMethod);
                if (branchPosition === 0) {
                    validBranches.push(branch);
                    break;
                }
                if (branchPosition === 1) {
                    methodsIndex++;
                }
            }
        }
        this._branchesArray = validBranches.sort(this.comparePosition);
    };
    /*
        this method checks if specific branch is inside specific method
        if inside method returns: 0
        if after method returns: 1
        if before method returns: -1
     */
    UniqueIdConverter.prototype.checkBranchPosition = function (branch, method) {
        if (branch.isStartsAfter(method) && branch.isEndsBefore(method)) {
            return 0;
        }
        if (branch.isStartsBefore(method) && branch.isEndsBefore(method)) {
            return -1;
        }
        if (branch.isStartsAfter(method) && branch.isEndsAfter(method)) {
            return 1;
        }
        this.logger.warn("branch started at '" + branch.startPosition.line + "," + branch.startPosition.column + "' in file '" + this._filename + "' located across methods, probably some error");
        return 0;
    };
    UniqueIdConverter.prototype.aggregateElementsByLine = function (elements) {
        var aggregated = {};
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            var lineNumber = element.startPosition.line;
            if (!aggregated[lineNumber]) {
                aggregated[lineNumber] = [];
            }
            aggregated[lineNumber].push(i);
        }
        return aggregated;
    };
    return UniqueIdConverter;
}());
exports.UniqueIdConverter = UniqueIdConverter;
//# sourceMappingURL=unique-id-converter.js.map