"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var validation_utils_1 = require("../utils/validation-utils");
var EventsProcess = /** @class */ (function () {
    function EventsProcess(configuration, sendToServerWatchdog, keepAliveWatchdog, environmentDataService, agentDataService, backendProxy, eventsQueue, logger) {
        this.isRunning = false;
        this.isSubmittingEvents = false;
        this.sequence = 0;
        this.stopCallbacks = [];
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(configuration, "configuration");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(sendToServerWatchdog, "sendToServerWatchdog");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(keepAliveWatchdog, "keepAliveWatchdog");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(environmentDataService, "environmentData");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(agentDataService, "agentDataService");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(backendProxy, "backendProxy");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(eventsQueue, "eventsQueue");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(logger, "logger");
        this.configuration = configuration;
        this.sendToServerWatchdog = sendToServerWatchdog;
        this.keepAliveWatchdog = keepAliveWatchdog;
        this.environmentData = environmentDataService;
        this.agentDataService = agentDataService;
        this.backendProxy = backendProxy;
        this.logger = logger;
        this.eventsQueue = eventsQueue;
        this.init();
    }
    EventsProcess.prototype.enqueueEvent = function (evt) {
        evt.localTime = evt.localTime || new Date().valueOf();
        this.eventsQueue.enqueue(evt);
        if (this.isRunning && this.configuration.sendEvents.value && this.configuration.enabled.value) {
            this.ensureKeepaliveThreadRunning();
        }
    };
    EventsProcess.prototype.start = function () {
        if (this.isRunning || !this.configuration.enabled.value) {
            return;
        }
        this.sendToServerWatchdog.start();
        if (this.eventsQueue.getQueueSize() > 0) {
            this.ensureKeepaliveThreadRunning();
        }
        this.isRunning = true;
    };
    EventsProcess.prototype.stop = function (callback) {
        var _this = this;
        if (callback === void 0) { callback = function () { }; }
        try {
            this.sendToServerWatchdog.stop();
            if (this.configuration.enabled.value === false || this.configuration.sendEvents.value === false) { //Events/Agent functionality is disabled. Return immediately
                this.isRunning = false;
                return callback();
            }
            this.submitEvents(false);
            if (this.isSubmittingEvents) {
                /*
                    Nothing to do right now, since events are being submitted
                    When the submission is over (successfully or not), if the queue is empty the next batch of events will be sent.
                    If after submission the queue is cleared, this callback will be called
                */
                this.stopCallbacks.push(function () {
                    _this.runStopCallback(callback);
                });
            }
            else {
                /*
                    Queue must be empty, and nothing is sent. submitQueuedEvents() would have started sending events if the queue was non-empty
                */
                return this.runStopCallback(callback);
            }
            this.isRunning = false;
        }
        catch (e) {
            this.logger.warn("Error while stopping EventsProcess: ", e);
            callback();
        }
    };
    EventsProcess.prototype.runStopCallback = function (callback) {
        this.isRunning = false;
        this.keepAliveWatchdog.stop();
        return callback();
    };
    EventsProcess.prototype.getQueueSize = function () {
        return this.eventsQueue.getQueueSize();
    };
    EventsProcess.prototype.submitEvents = function (async) {
        var _this = this;
        if (async === void 0) { async = true; }
        if (!this.isRunning || !this.configuration.sendEvents.value || !this.configuration.enabled.value) {
            return;
        }
        if ((this.isSubmittingEvents && async) || this.eventsQueue.getQueueSize() == 0) {
            return;
        }
        var items = this.eventsQueue.dequeue(EventsProcess.ITEMS_TO_DEQUE);
        var packet = this.createEventsPacket(items);
        this.isSubmittingEvents = true;
        this.backendProxy.submitEvents(packet, function (err, body) {
            _this.isSubmittingEvents = false;
            if (err) {
                _this.logger.warn("Error while submitting events, " + err);
                _this.eventsQueue.requeue(items);
            }
            else if (_this.eventsQueue.getQueueSize() > 0) {
                _this.submitEvents();
            }
            else {
                _this.fireQueueEmpty();
            }
        }, async);
    };
    EventsProcess.prototype.createEventsPacket = function (items) {
        return {
            customerId: this.configuration.customerId.value,
            appName: this.configuration.appName.value,
            build: this.configuration.build.value,
            branch: this.configuration.branch.value,
            environment: this.environmentData,
            configurationData: this.configuration.toJsonObject,
            events: items,
            meta: {
                sequence: ++this.sequence,
                generated: new Date().valueOf(),
                agentId: this.agentDataService.getAgentId(),
            },
        };
    };
    EventsProcess.prototype.init = function () {
        var _this = this;
        this.sendToServerWatchdog.on("alarm", function () { return _this.submitEvents(); });
        this.eventsQueue.on("full", function () { return _this.submitEvents(); });
        this.keepAliveWatchdog.on("alarm", function () {
            if (!_this.isSubmittingEvents && _this.eventsQueue.getQueueSize() == 0) {
                _this.keepAliveWatchdog.stop();
            }
        });
    };
    EventsProcess.prototype.ensureKeepaliveThreadRunning = function () {
        this.keepAliveWatchdog.start();
    };
    EventsProcess.prototype.fireQueueEmpty = function () {
        var _this = this;
        if (this.stopCallbacks.length > 0) {
            this.stopCallbacks.forEach(function (cb) {
                try {
                    cb();
                }
                catch (err) {
                    _this.logger.error("Error while calling stopCallbacks: ", err);
                }
            });
        }
    };
    EventsProcess.ITEMS_TO_DEQUE = 1000;
    return EventsProcess;
}());
exports.EventsProcess = EventsProcess;
//# sourceMappingURL=index.js.map