"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var queue_1 = require("../queue");
var FootprintsProcess = /** @class */ (function () {
    function FootprintsProcess(
    /**
     * Initial configuration
     */
    cfg, 
    /**
     * Background thread that tells the footprints service to send footprints from the queue to the server
     */
    sendToServerWatchdog, 
    /**
     * Foreground thread that prevents the process from shutting down, and kept alive while there are items to submit to the server or items are currently being submitted
     */
    keepaliveWatchdog, 
    /**
     * Proxy to the server for footprints submission
     */
    footprintsServiceProxy, 
    /**
     * Environments service, used to get environmental data for submission (process id, machine data, etc.)
     */
    environmentDataService, agentDataService, istanbulToFootprints, logger) {
        var _this = this;
        this.cfg = cfg;
        this.sendToServerWatchdog = sendToServerWatchdog;
        this.keepaliveWatchdog = keepaliveWatchdog;
        this.footprintsServiceProxy = footprintsServiceProxy;
        this.environmentDataService = environmentDataService;
        this.agentDataService = agentDataService;
        this.istanbulToFootprints = istanbulToFootprints;
        this.logger = logger;
        this.isSubmittingFootprints = false;
        this.sequence = 0;
        this.isRunning = false;
        this.stopCallbacks = [];
        if (!cfg)
            throw new Error("cfg was not specified");
        if (!sendToServerWatchdog)
            throw new Error("sendToServerWatchdog was not specified");
        if (!keepaliveWatchdog)
            throw new Error("keepaliveWatchdog was not specified");
        if (!footprintsServiceProxy)
            throw new Error("footprintsServiceProxy was not specified");
        if (!environmentDataService)
            throw new Error("environmentDataService was not specified");
        if (!agentDataService)
            throw new Error("agentDataService was not specified");
        if (!istanbulToFootprints)
            throw new Error("istanbulToFootprints was not specified");
        if (!logger) {
            throw new Error("logger was not specified");
        }
        this.footprintsQueue = new queue_1.Queue();
        sendToServerWatchdog.on("alarm", function () {
            _this.logger.debug("Start submitting footprints, triggered by send to server watchdog.");
            _this.submitQueuedFootprints();
        });
        this.footprintsQueue.on("full", function () {
            _this.logger.debug("Start submitting footprints, queue is full.");
            _this.submitQueuedFootprints();
        });
        keepaliveWatchdog.on("alarm", function () {
            //console.log('keepaliveWatchdog. isSubmittingFootprints:' + this.isSubmittingFootprints + ', queue size: ' + this.footprintsQueue.getQueueSize());
            if (!_this.isSubmittingFootprints && _this.footprintsQueue.getQueueSize() == 0) {
                keepaliveWatchdog.stop();
            }
        });
    }
    FootprintsProcess.prototype.enqueueCurrentFootprints = function (executionId, testName) {
        var footprints = this.istanbulToFootprints.getCurrentFootprints();
        var hitModules = Object.keys(footprints);
        if (!hitModules || hitModules.length == 0) { //If no modules were hit at all, don't enqueue anything
            this.logger.info("No files were hit, not enqueuing footprints.");
            return;
        }
        this.logger.info("Enqueuing footprints for %s files.", hitModules.length);
        this.footprintsQueue.enqueue({
            executionId: executionId,
            testName: testName,
            footprints: footprints,
            localTime: new Date().valueOf()
        });
        if (this.isRunning && this.cfg.sendFootprints.value && this.cfg.enabled.value) {
            this.ensureKeepaliveThreadRunning();
            this.sendToServerWatchdog.start();
        }
    };
    FootprintsProcess.prototype.ensureKeepaliveThreadRunning = function () {
        this.keepaliveWatchdog.start();
    };
    FootprintsProcess.prototype.updateConfig = function (updatedCfg) {
        if (updatedCfg.sendFootprints.value === false || updatedCfg.enabled.value === false) {
            this.footprintsQueue.clear();
            this.stop(function () { });
        }
        this.cfg = updatedCfg;
        this.sendToServerWatchdog.setInterval(this.cfg.interval.value);
    };
    FootprintsProcess.prototype.createFootprintsPacket = function (items) {
        var packet = this.istanbulToFootprints.createFootprintsPackageFromQueue(items, {
            customerId: this.cfg.customerId.value,
            environment: this.environmentDataService.getEnvDataForSubmission(),
            configurationData: this.cfg.toJsonObject(),
            meta: {
                sequence: ++this.sequence,
                generated: new Date().valueOf(),
                agentId: this.agentDataService.getAgentId(),
            },
            tests: [],
            apps: []
        });
        return packet;
    };
    FootprintsProcess.prototype.submitQueuedFootprints = function () {
        var _this = this;
        if (!this.isRunning) {
            this.logger.info("Agent is not running, not sending footprints");
            return;
        }
        if (this.cfg.sendFootprints.value === false) {
            this.logger.info("Not sending footprints since agent is configured to not send.");
            return; //Can't start without server or token
        }
        if (this.isSubmittingFootprints) {
            this.logger.info("Not sending footprints since its already submitting.");
            return;
        }
        if (this.footprintsQueue.getQueueSize() == 0) {
            this.logger.info("Not sending footprints since queue size is 0");
            return;
        }
        this.logger.info("About to send footprints. Queue size:" + this.footprintsQueue.getQueueSize());
        var items = this.footprintsQueue.dequeue(/*this.cfg.maxFootprintsChunkSize ||*/ 1000);
        var packet = this.createFootprintsPacket(items);
        this.isSubmittingFootprints = true;
        this.footprintsServiceProxy.handleFootprints(packet, function (err) {
            _this.isSubmittingFootprints = false;
            _this.logger.info("Sent footprints. Queue size:" + _this.footprintsQueue.getQueueSize() + ", err:" + err);
            if (err) {
                _this.environmentDataService.setLastError(err.stack);
                _this.footprintsQueue.requeue(items); //Requeue items for later submission
            }
            else if (_this.footprintsQueue.getQueueSize() > 0) { //If there was an error, don't send now. Otherwise, if the queue still has items, send those now
                _this.submitQueuedFootprints();
            }
            else
                _this.fireQueueEmpty();
        });
    };
    FootprintsProcess.prototype.fireQueueEmpty = function () {
        var _this = this;
        if (this.stopCallbacks.length > 0) {
            this.stopCallbacks.forEach(function (cb) {
                try {
                    cb();
                }
                catch (err) {
                    _this.environmentDataService.setLastError(err.stack);
                }
            });
        }
    };
    FootprintsProcess.prototype.start = function () {
        if (this.isRunning)
            return; //Already started
        if (this.cfg.enabled.value == false)
            return; //Don't do anything if agent is completely disabled
        this.isRunning = true;
        this.sendToServerWatchdog.start();
        if (this.footprintsQueue.getQueueSize() > 0) {
            this.ensureKeepaliveThreadRunning();
        }
    };
    FootprintsProcess.prototype.stop = function (callback) {
        var _this = this;
        try {
            this.sendToServerWatchdog.stop();
            if (this.cfg.enabled.value === false || this.cfg.sendFootprints.value === false) { //Footprints/Agent functionality is disabled. Return immediately
                this.isRunning = false;
                return callback();
            }
            this.logger.debug("Start submitting footprints, triggered by stop event.");
            this.submitQueuedFootprints(); //If footprints are being submitted, this has no effect. Otherwise, it will start submitting footprints if the queue is non-empty
            if (this.isSubmittingFootprints) {
                /*
                    Nothing to do right now, since footprints are being submitted
                    When the submission is over (successfully or not), if the queue is empty the next batch of footprints will be sent.
                    If after submission the queue is cleared, this callback will be called
                */
                this.stopCallbacks.push(function () {
                    _this.isRunning = false;
                    _this.keepaliveWatchdog.stop();
                    return callback();
                });
            }
            else {
                /*
                    Queue must be empty, and nothing is sent. submitQueuedFootprints() would have started sending footprints if the queue was non-empty
                */
                this.isRunning = false;
                this.keepaliveWatchdog.stop();
                return callback();
            }
        }
        catch (err) {
            this.logger.error(err);
            return callback();
        }
    };
    FootprintsProcess.prototype.getQueueSize = function () {
        return this.footprintsQueue.getQueueSize();
    };
    return FootprintsProcess;
}());
exports.FootprintsProcess = FootprintsProcess;
//# sourceMappingURL=index.js.map