"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var messages_formatter_1 = require("../../lib/messages-formatter");
var original_module_loader_1 = require("../coverage-elements/original-module-loader");
var files_utils_1 = require("../utils/files-utils");
var source_maps_utils_1 = require("../source-maps-utils");
var short_name_to_file_mapper_1 = require("../short-name-to-file-mapper");
var sl_env_vars_1 = require("../constants/sl-env-vars");
var istanbul_unique_id_converter_1 = require("../coverage-elements/istanbul-unique-id-converter");
var source_map_handler_1 = require("../source-map-handler");
var GLOBAL_ISTANBUL_CONTAINER_NAMES = ["__coverage__"];
var IstanbulToFootprints = /** @class */ (function () {
    function IstanbulToFootprints(cfg, logger) {
        this.cfg = cfg;
        this.totalTests = 0;
        this.previousIstanbulCounters = {};
        this.relativeModulePaths = {};
        if (!logger)
            throw new Error("logger is required");
        if (!cfg)
            throw new Error("cfg is required");
        this.logger = logger;
    }
    IstanbulToFootprints.prototype.resetState = function () {
        this.totalTests = 0;
        //Holds the a mapping between a uniqueId for an element in a test, to an index in this elements 'hits' array.
        this.eidToHitsIndex = {};
        this.uniqueIdToElement = {};
        this.testNameToTestData = {};
        this.fileNameToAppFile = {};
        this.fileToSourceMapConsumer = {};
    };
    IstanbulToFootprints.prototype.createFootprintsPackageFromQueue = function (queuedFootprints, extraData) {
        var _this = this;
        this.resetState();
        var result = this.createFootprintsAppData(extraData);
        var istanbulMaps = this.getIstanbulCounters();
        if (sl_env_vars_1.SlEnvVars.isUseNewUniqueId()) {
            this.loadOriginalModules();
            this.mapShortNamesToFiles();
        }
        queuedFootprints.forEach(function (fp) {
            _this.convertIstanbulDataToFootprints(fp, result, istanbulMaps);
        });
        return result;
    };
    IstanbulToFootprints.prototype.createFootprintsPackageFromRawData = function (rawData, extraData) {
        this.resetState();
        this.nycFinalReportData = rawData.footprints;
        var result = this.createFootprintsAppData(extraData);
        if (sl_env_vars_1.SlEnvVars.isUseNewUniqueId()) {
            this.loadOriginalModules();
            this.mapShortNamesToFiles();
        }
        this.convertIstanbulDataToFootprints(rawData, result);
        return result;
    };
    IstanbulToFootprints.prototype.convertIstanbulDataToFootprints = function (footprintsElement, result, istanbulMaps) {
        var test = this.getOrCreateTestData(footprintsElement.testName, footprintsElement.executionId, footprintsElement.localTime, result);
        var app = result.apps[0]; // We assume single App for now.
        var fullIstanbulData = istanbulMaps || footprintsElement.footprints;
        for (var moduleName in footprintsElement.footprints) {
            var istanbulModule = footprintsElement.footprints[moduleName];
            var moduleWithMaps = fullIstanbulData[moduleName];
            this.logger.debug("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
            this.logger.debug("moduleName: ", moduleName);
            if (!moduleWithMaps) {
                this.logger.info("Cannot find full istanbul data for %s", moduleName);
                continue;
            }
            var relativePath = this.getRelativeModulePath(moduleWithMaps);
            var sourceMap_1 = this.getSourceMapForModule(moduleName);
            var file = this.getOrCreateFootprintsAppFile(relativePath, app);
            if (istanbulModule.f) {
                this.handleFunctionsData(istanbulModule, moduleWithMaps, sourceMap_1, relativePath, moduleWithMaps.path, file, test);
            }
            if (istanbulModule.b) {
                this.handleBranchesData(istanbulModule, moduleWithMaps, sourceMap_1, relativePath, moduleWithMaps.path, file, test);
            }
        }
    };
    IstanbulToFootprints.prototype.handleBranchesData = function (istanbulModule, moduleWithMaps, sourceMap, relPath, fullPath, file, test) {
        for (var branchId in istanbulModule.b) {
            var ifElseAndSwitchCasesArr = istanbulModule.b[branchId];
            var expressionInfo = moduleWithMaps.branchMap[branchId];
            for (var idx = 0; idx < ifElseAndSwitchCasesArr.length; idx++) {
                var branchInfo = expressionInfo.locations[idx];
                var hits = ifElseAndSwitchCasesArr[idx];
                if (hits > 0) {
                    var start = branchInfo.start;
                    var sourceMapData = this.readSourceMapData(sourceMap, start, relPath, fullPath);
                    var filePath = relPath;
                    var fullFilePath = fullPath;
                    if (sourceMapData) {
                        filePath = sourceMapData.relativePath;
                        start = sourceMapData.originalStart;
                        fullFilePath = sourceMapData.absolutePath;
                    }
                    var uniqueId = filePath + "|" + this.formatLoc(start) + "|" + idx;
                    if (sl_env_vars_1.SlEnvVars.isUseNewUniqueId()) {
                        var shortPath = this.shortNameToFileMapper.resolveShortNameForFile(fullFilePath);
                        var uniqueIdKey = shortPath + "|" + this.formatLoc(start) + "|" + idx;
                        uniqueId = this.getUniqueId(uniqueIdKey, fullFilePath);
                    }
                    if (!uniqueId) {
                        continue;
                    }
                    var branchElement = this.getOrCreateBranchElement(uniqueId, file);
                    if (this.cfg.extendedFootprints.value) {
                        branchElement.branchInfo = JSON.stringify(branchInfo);
                        branchElement.sourceMapData = JSON.stringify(sourceMapData);
                    }
                    this.addOrUpdateElementHits(test, uniqueId, branchElement, hits);
                }
            }
        }
    };
    IstanbulToFootprints.prototype.handleFunctionsData = function (istanbulModule, moduleWithMaps, sourceMap, relativePath, fullPath, file, test) {
        for (var funcId in istanbulModule.f) {
            var hits = istanbulModule.f[funcId];
            if (hits > 0) {
                var funcInfo = moduleWithMaps.fnMap[funcId];
                var start = funcInfo.loc.start;
                var sourceMapData = this.readSourceMapData(sourceMap, start, relativePath, fullPath);
                var filePath = relativePath;
                var fullFilePath = fullPath;
                if (sourceMapData) {
                    filePath = sourceMapData.relativePath;
                    start = sourceMapData.originalStart;
                    fullFilePath = sourceMapData.absolutePath;
                }
                var uniqueId = filePath + "@" + this.formatLoc(start);
                if (sl_env_vars_1.SlEnvVars.isUseNewUniqueId()) {
                    var shortPath = this.shortNameToFileMapper.resolveShortNameForFile(fullFilePath);
                    var uniqueIdKey = shortPath + "@" + this.formatLoc(start);
                    uniqueId = this.getUniqueId(uniqueIdKey, fullFilePath);
                }
                if (!uniqueId) {
                    continue;
                }
                var methodElement = this.getOrCreateMethodElement(uniqueId, file);
                if (this.cfg.extendedFootprints.value) {
                    methodElement.funcInfo = JSON.stringify(funcInfo);
                    methodElement.sourceMapData = JSON.stringify(sourceMapData);
                }
                this.addOrUpdateElementHits(test, uniqueId, methodElement, hits);
                if (funcInfo.decl) {
                    this.addDeclarationFootprints(funcInfo, sourceMap, relativePath, file, fullPath, test, hits);
                }
            }
        }
    };
    IstanbulToFootprints.prototype.addDeclarationFootprints = function (funcInfo, sourceMap, relativePath, file, fullpath, test, hits) {
        var declStart = funcInfo.decl.start; //For NYC
        var sourceMapData = this.readSourceMapData(sourceMap, declStart, relativePath, fullpath);
        if (sourceMapData) {
            relativePath = sourceMapData.relativePath;
            declStart = sourceMapData.originalStart;
        }
        var declUniqueId = relativePath + "@" + this.formatLoc(declStart);
        /*
            There is some edge case where 'loc' object of one method is same as 'decl' object of another method, and
            this can lead to missing coverage for first method when using new uniqueIds. because declaration key of
            second method already points to the first method so we add declaration footprints also.
            The cost effect is that hit of every method will doubled. (one for 'loc' and one for 'decl').
         */
        if (sl_env_vars_1.SlEnvVars.isUseNewUniqueId()) {
            var shortPath = this.shortNameToFileMapper.resolveShortNameForFile(fullpath);
            var uniqueIdKey = shortPath + "@" + this.formatLoc(declStart);
            declUniqueId = this.getUniqueId(uniqueIdKey, fullpath);
        }
        if (!declUniqueId) {
            return;
        }
        var declMethodElement = this.getOrCreateMethodElement(declUniqueId, file);
        this.addOrUpdateElementHits(test, declUniqueId, declMethodElement, hits);
    };
    IstanbulToFootprints.prototype.getSourceMapForModule = function (relPath) {
        if (!this.fileToSourceMapConsumer[relPath]) {
            this.fileToSourceMapConsumer[relPath] = source_maps_utils_1.SourceMapsUtils.readSourceMaps(relPath);
            this.logger.debug("Read source maps for file':" + relPath + "'");
        }
        var sourceMap = this.fileToSourceMapConsumer[relPath];
        return sourceMap;
    };
    IstanbulToFootprints.prototype.addOrUpdateElementHits = function (testData, elementUniqueId, element, hits) {
        var HITS_INDEX = 1;
        var eid = elementUniqueId + "|" + testData.index;
        var index = this.eidToHitsIndex[eid];
        if (index != null) {
            var testIndexAndHits = element.hits[index];
            var elementHits = testIndexAndHits[HITS_INDEX];
            elementHits = elementHits + hits;
            testIndexAndHits[HITS_INDEX] = elementHits;
        }
        else {
            var testIndexAndHits = new Array();
            testIndexAndHits.push(testData.index);
            testIndexAndHits.push(hits);
            var length = element.hits.push(testIndexAndHits);
            this.eidToHitsIndex[eid] = length - 1;
        }
    };
    IstanbulToFootprints.prototype.getOrCreateTestData = function (testName, executionId, localTime, result) {
        if (!this.testNameToTestData[testName]) {
            var test_1 = {
                executionId: executionId,
                localTime: localTime,
                testName: testName
            };
            result.tests.push(test_1);
            this.testNameToTestData[testName] = {
                index: this.totalTests++,
                test: test_1
            };
        }
        return this.testNameToTestData[testName];
    };
    IstanbulToFootprints.prototype.getOrCreateMethodElement = function (uniqueId, file) {
        if (!this.uniqueIdToElement[uniqueId]) {
            var x = {
                hits: new Array(),
                uniqueId: uniqueId
            };
            this.uniqueIdToElement[uniqueId] = x;
            file.methods.push(x);
        }
        return this.uniqueIdToElement[uniqueId];
    };
    IstanbulToFootprints.prototype.getOrCreateBranchElement = function (uniqueId, file) {
        if (!this.uniqueIdToElement[uniqueId]) {
            var x = {
                hits: new Array(),
                uniqueId: uniqueId
            };
            this.uniqueIdToElement[uniqueId] = x;
            file.branches.push(x);
        }
        return this.uniqueIdToElement[uniqueId];
    };
    IstanbulToFootprints.prototype.getOrCreateFootprintsAppFile = function (fileName, app) {
        if (this.fileNameToAppFile[fileName]) {
            return this.fileNameToAppFile[fileName];
        }
        //File doesn't exist in request.
        var file = {
            methods: new Array(),
            branches: new Array(),
            lines: new Array(),
            path: fileName
        };
        app.files.push(file);
        this.fileNameToAppFile[fileName] = file;
        return file;
    };
    IstanbulToFootprints.prototype.createFootprintsAppData = function (extraData) {
        //Supporting only one app right now
        var app = {
            appName: this.cfg.appName.value,
            branchName: this.cfg.branch.value,
            buildName: this.cfg.build.value,
            moduleName: null,
            files: new Array()
        };
        extraData.apps.push(app);
        return extraData;
    };
    IstanbulToFootprints.prototype.readSourceMapData = function (sourceMaps, start, coveragePath, fullPath) {
        if (sourceMaps) {
            var startPositionArr = [start.line, start.column];
            var handler = new source_map_handler_1.SourceMapHandler(sourceMaps, fullPath, startPositionArr, [], this.cfg.projectRoot.value, this.logger);
            handler.resolvePaths();
            if (!handler.shouldSkipped) {
                var result = {
                    relativePath: handler.relativePath,
                    originalStart: handler.start,
                    absolutePath: handler.absolutePath
                };
                return result;
            }
        }
        return null;
    };
    IstanbulToFootprints.prototype.adjustPathSlashes = function (filePath) {
        filePath = (filePath || "").replace(/\\/g, '/');
        return filePath;
    };
    IstanbulToFootprints.prototype.getIstanbulCounters = function () {
        if (this.globalCoverageContainer)
            return this.globalCoverageContainer;
        var re = /^\$\$cov_[0-9]+\$\$$/;
        var keys = Object.keys(global);
        var m;
        for (var i = 0; i < keys.length; i++) {
            var k = keys[i];
            if ((m = re.exec(k)) !== null || GLOBAL_ISTANBUL_CONTAINER_NAMES.indexOf(k) >= 0) {
                this.globalCoverageContainer = global[k];
                return this.globalCoverageContainer;
            }
        }
        return null;
    };
    IstanbulToFootprints.prototype.getCoverageObject = function () {
        return this.nycFinalReportData || this.getIstanbulCounters();
    };
    IstanbulToFootprints.prototype.getRelativeModulePath = function (module) {
        // Sometimes istanbul holds the 'path' under 'data' object. see coverage final attached in 'https://sealights.atlassian.net/browse/SLDEV-4421'
        if (typeof (module.path) !== "string" && module.data.path && typeof (module.data.path) === "string") {
            this.logger.debug("Not extracting relative path for '" + module.data.path + "'. Using the same path as is.");
            return module.data.path;
        }
        if (!this.relativeModulePaths[module.path]) {
            var absolutePath = this.adjustPathSlashes(module.path);
            var relativePath = absolutePath.replace(this.resolveProjectRoot(), "");
            if (relativePath.indexOf("/") === 0) {
                relativePath = relativePath.substring(1);
            }
            this.relativeModulePaths[module.path] = relativePath;
        }
        return this.relativeModulePaths[module.path];
    };
    IstanbulToFootprints.prototype.formatLoc = function (loc) {
        return loc.line + ',' + loc.column;
    };
    IstanbulToFootprints.prototype.getCurrentFootprints = function () {
        var diff = this.getIstanbulCountersDiff(this.getIstanbulCounters());
        return diff;
    };
    IstanbulToFootprints.prototype.getIstanbulCountersDiff = function (currentCounters) {
        if (!currentCounters) {
            this.logger.error("Couldn't get istanbul counters, return empty diff.");
            return {};
        }
        var diff = {};
        var diffModule = null;
        function createModuleForDiff(istanbulModule) {
            diffModule = diff[istanbulModule] = { b: {}, f: {}, s: {} };
            return diffModule;
        }
        var prevCounters = this.previousIstanbulCounters;
        for (var istanbulModule in currentCounters) {
            diffModule = null;
            if (!prevCounters[istanbulModule]) {
                prevCounters[istanbulModule] = { b: {}, f: {}, s: {} };
            }
            var prevModule = prevCounters[istanbulModule];
            for (var funcId in currentCounters[istanbulModule].f) {
                if (prevModule.f[funcId] === undefined) {
                    prevModule.f[funcId] = 0;
                }
                var currentFunctionHits = currentCounters[istanbulModule].f[funcId];
                var prevFunctionHits = prevModule.f[funcId] || 0;
                if (currentFunctionHits > prevFunctionHits) {
                    //this is a diff
                    diffModule = diffModule || createModuleForDiff(istanbulModule);
                    diffModule.f[funcId] = (currentFunctionHits - prevFunctionHits);
                    prevModule.f[funcId] = currentFunctionHits;
                }
            }
            for (var branchId in currentCounters[istanbulModule].b) {
                var currentBranchesHits = currentCounters[istanbulModule].b[branchId];
                var prevBranchHits = prevModule.b[branchId];
                if (prevBranchHits) {
                    for (var i = 0, j = currentBranchesHits.length; i < j; i++) {
                        var currentHits = currentBranchesHits[i], previousHits = prevBranchHits[i] || 0;
                        if (currentHits > previousHits) {
                            //this is a diff
                            diffModule = diffModule || createModuleForDiff(istanbulModule);
                            diffModule.b[branchId] = diffModule.b[branchId] || this.newNumberArray(currentBranchesHits.length);
                            diffModule.b[branchId][i] = currentHits - previousHits;
                            prevBranchHits[i] = currentHits;
                        }
                    }
                }
                else {
                    prevModule.b[branchId] = currentBranchesHits.slice();
                    var branchSetHasHits = false;
                    for (var i = 0, j = currentBranchesHits.length; i < j && !branchSetHasHits; i++) {
                        if (currentBranchesHits[i])
                            branchSetHasHits = true;
                    }
                    if (branchSetHasHits) {
                        diffModule = diffModule || createModuleForDiff(istanbulModule);
                        diffModule.b[branchId] = currentBranchesHits.slice();
                    }
                }
            }
        }
        this.logDiff(diff);
        return diff;
    };
    IstanbulToFootprints.prototype.logDiff = function (diff) {
        if (!this.logger.debug()) {
            return;
        }
        var modules = Object.keys(diff);
        var formatter = new messages_formatter_1.MessagesFormatter();
        modules.forEach(function (module) {
            var methods = diff[module].f || {};
            var branches = diff[module].b || {};
            var methodsSize = Object.keys(methods).length;
            var branchesSize = Object.keys(branches).length;
            formatter.addContent(module + ": " + methodsSize + " methods and " + branchesSize + " branches");
        });
        this.logger.debug("Found coverage data for %s files:\n %s", modules.length, formatter.content);
    };
    IstanbulToFootprints.prototype.newNumberArray = function (length) {
        var arr = [];
        for (var i = 0; i < length; i++)
            arr.push(0);
        return arr;
    };
    IstanbulToFootprints.prototype.getUniqueId = function (uniqueIdKey, moduleName) {
        var coverageObject = this.getCoverageObject();
        moduleName = path.normalize(moduleName);
        var moduleData = this.resolveModule(coverageObject, moduleName);
        if (!moduleData) {
            return;
        }
        if (!moduleData.uniqueIdsMap) {
            this.logger.debug("building uniqueids map for " + moduleData.path);
            var before = new Date().getTime();
            this.logger.debug("file " + moduleData.path + " not contains uniqueIdMaps, trying to reload");
            this.createSLMapping(moduleData.path);
            var after = new Date().getTime();
            this.logger.debug("*******************************create unique id map ******************************************");
            this.logger.debug(after - before);
            this.logger.debug("*******************************create unique id map ******************************************");
        }
        var newUniqueId = moduleData.uniqueIdsMap[uniqueIdKey];
        if (!newUniqueId) {
            this.logger.error("could not generate new uniqueId for '" + uniqueIdKey + "'");
        }
        return newUniqueId;
    };
    IstanbulToFootprints.prototype.resolveModule = function (coverageObject, moduleName) {
        var withLeftSlashes = files_utils_1.FilesUtils.adjustPathSlashes(moduleName);
        var moduleData = coverageObject[moduleName] || coverageObject[withLeftSlashes];
        if (!moduleData) {
            this.logger.warn("Coverage object not contains data for " + moduleName + " or " + withLeftSlashes);
        }
        return moduleData;
    };
    IstanbulToFootprints.prototype.createSLMapping = function (fullPath) {
        var shortPath = this.shortNameToFileMapper.resolveShortNameForFile(fullPath);
        var istanbulData = this.getCoverageObject();
        var module = istanbulData[fullPath];
        var converter = new istanbul_unique_id_converter_1.IstanbulUniqueIdConverter(shortPath, this.logger);
        converter.createElementsArray(module);
        converter.process();
        var coverageObject = this.getCoverageObject();
        coverageObject[fullPath].uniqueIdsMap = converter.uniqueIdsMap;
    };
    IstanbulToFootprints.prototype.loadOriginalModules = function () {
        var loader = new original_module_loader_1.OriginalModuleLoader(this.getCoverageObject(), this.logger);
        loader.load();
    };
    IstanbulToFootprints.prototype.mapShortNamesToFiles = function () {
        var _this = this;
        this.shortNameToFileMapper = new short_name_to_file_mapper_1.ShortNameToFileMapper(this.logger);
        Object.keys(this.getCoverageObject()).forEach(function (module) { return _this.shortNameToFileMapper.addFile(module); });
    };
    IstanbulToFootprints.prototype.resolveProjectRoot = function () {
        return (this.cfg.projectRoot.value || process.cwd()).replace(/\\/g, '/');
    };
    return IstanbulToFootprints;
}());
exports.IstanbulToFootprints = IstanbulToFootprints;
//# sourceMappingURL=istanbul-to-footprints.js.map