"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http_client_1 = require("./http-client");
var sl_routes_1 = require("./sl-routes");
var validation_utils_1 = require("../utils/validation-utils");
var entities_mapper_1 = require("./entities-mapper");
var constants_1 = require("../constants/constants");
var BackendProxy = /** @class */ (function () {
    // TODO: get httpClient in constructor always
    function BackendProxy(config, logger, httpClient) {
        this.config = config;
        this.logger = logger;
        this.client = httpClient || new http_client_1.HttpClient(this.config, this.logger);
    }
    BackendProxy.prototype.getBuildSession = function (buildSessionId, callback) {
        var _this = this;
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(buildSessionId, constants_1.Constants.BUILD_SESSION_ID);
        var url = sl_routes_1.SLRoutes.buildSessionV2(buildSessionId);
        this.client.get(url, function (err, body) {
            _this.invokeCallback(callback, err, body);
        });
    };
    BackendProxy.prototype.createBuildSessionId = function (request, callback) {
        var _this = this;
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(request, constants_1.Constants.REQUEST);
        var url = sl_routes_1.SLRoutes.buildSessionV2();
        this.client.post(request, url, function (err, body) {
            _this.invokeCallback(callback, err, body, entities_mapper_1.EntitiesMapper.toCreateBuildSessionIdResponse);
        });
    };
    BackendProxy.prototype.getRecommendedVersion = function (request) {
        var _this = this;
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(request, constants_1.Constants.REQUEST);
        var url = sl_routes_1.SLRoutes.recommendedAgentV2(request.componentName, request.customerId, request.appName, request.branchName, request.testStage);
        return new Promise(function (resolve, reject) {
            _this.client.get(url, function (err, body) {
                if (err) {
                    return reject(err);
                }
                else if (body && body.agent && body.agent) {
                    return resolve(body.agent);
                }
                else {
                    return reject(new Error("Failed to get recommended version due to unexpected response from the server."));
                }
            });
        });
    };
    BackendProxy.prototype.submitBuildMapping = function (request) {
        var _this = this;
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(request, constants_1.Constants.REQUEST);
        var url = sl_routes_1.SLRoutes.buildMappingV3();
        return new Promise(function (resolve, reject) {
            _this.client.post(request, url, function (err, body) {
                if (err) {
                    return reject(err);
                }
                return resolve(body);
            });
        });
    };
    BackendProxy.prototype.submitLogs = function (request, callback) {
        var _this = this;
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(request, constants_1.Constants.REQUEST);
        var url = sl_routes_1.SLRoutes.logSubmissionV2();
        this.client.post(request, url, function (err, body) {
            _this.invokeCallback(callback, err, body);
        });
    };
    BackendProxy.prototype.getRemoteConfig = function (request, callback) {
        var _this = this;
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(request, constants_1.Constants.REQUEST);
        var url = sl_routes_1.SLRoutes.configV2(request.component, request.customerId, request.appName, request.branch, request.testStage, request.version);
        this.client.get(url, function (err, body) {
            _this.invokeCallback(callback, err, body, entities_mapper_1.EntitiesMapper.toGetRemoteConfigResponse);
        });
    };
    BackendProxy.prototype.startExecution = function (request, callback) {
        var _this = this;
        var url = sl_routes_1.SLRoutes.testExecution();
        this.client.post(request, url, function (err, body) {
            _this.invokeCallback(callback, err, body);
        });
    };
    BackendProxy.prototype.uploadReport = function (request, callback) {
        var _this = this;
        var url = sl_routes_1.SLRoutes.externalData();
        this.client.postMultipart(request, url, function (err, body) {
            _this.invokeCallback(callback, err, body);
        });
    };
    BackendProxy.prototype.externalReport = function (request, callback) {
        var _this = this;
        var url = sl_routes_1.SLRoutes.externalReport();
        this.client.post(request, url, function (err, body) {
            _this.invokeCallback(callback, err, body);
        });
    };
    BackendProxy.prototype.endExecution = function (request, callback) {
        var _this = this;
        var url = sl_routes_1.SLRoutes.endExecution(request.customerId, request.appName, request.buildName, request.branchName, request.environment);
        this.client.delete(url, function (err, body) {
            _this.invokeCallback(callback, err, body);
        });
    };
    BackendProxy.prototype.submitEvents = function (packetToSend, callback, async) {
        if (async === void 0) { async = true; }
        var url = sl_routes_1.SLRoutes.eventsV2();
        this.client.post(packetToSend, url, callback);
    };
    BackendProxy.prototype.submitBlob = function (body, buildSessionId, blobId, callback) {
        var url = sl_routes_1.SLRoutes.blobs(buildSessionId, blobId);
        this.client.post(body, url, callback, true, "application/octet-stream");
    };
    BackendProxy.prototype.invokeCallback = function (callback, err, body, map) {
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(callback, constants_1.Constants.CALLBACK);
        if (err != null) {
            return callback(err, body);
        }
        if (map == null) {
            return callback(null, body);
        }
        var apiResponse = map(body);
        return callback(null, apiResponse);
    };
    return BackendProxy;
}());
exports.BackendProxy = BackendProxy;
//# sourceMappingURL=backend-proxy.js.map