"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var request = require("request");
var zlib = require("zlib");
var uuid = require("uuid/v1");
var sl_env_vars_1 = require("../constants/sl-env-vars");
var http_verb_1 = require("./http-verb");
var validation_utils_1 = require("../utils/validation-utils");
var DEFAULT_CONTENT_TYPE = "application/json";
var HttpClient = /** @class */ (function () {
    function HttpClient(cfg, logger) {
        this.cfg = cfg;
        this.logger = logger;
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(cfg, "cfg");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(logger, "logger");
        this.defaultTimeout = this.getDefaultTimeout();
    }
    HttpClient.prototype.get = function (urlPath, callback) {
        this.invokeHttpRequest(http_verb_1.HttpVerb.GET, urlPath, callback);
    };
    HttpClient.prototype.delete = function (urlPath, callback) {
        this.invokeHttpRequest(http_verb_1.HttpVerb.DELETE, urlPath, callback);
    };
    HttpClient.prototype.post = function (requestData, urlPath, callback, async, contentType) {
        var _this = this;
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(requestData, "requestData");
        var bufferToSend = new Buffer(JSON.stringify(requestData));
        var shouldZip = this.cfg.compressRequests != null ? this.cfg.compressRequests : true;
        this.logger.debug("Sending buffer:" + bufferToSend.toString());
        if (shouldZip) {
            zlib.gzip(bufferToSend, function (err, compressedBuf) {
                if (err) {
                    //Fallback to sending data uncompressed
                    _this.logger.warn("Failed while trying to compress the request data. Sending uncompressed data instead. Error: ", err);
                    _this.invokeHttpRequest(http_verb_1.HttpVerb.POST, urlPath, callback, null, bufferToSend, contentType);
                }
                else {
                    _this.invokeHttpRequest(http_verb_1.HttpVerb.POST, urlPath, callback, { "Content-Encoding": "gzip" }, compressedBuf, contentType);
                }
            });
        }
        else {
            this.invokeHttpRequest(http_verb_1.HttpVerb.POST, urlPath, callback, null, bufferToSend, contentType); //Submit buffer as-is
        }
    };
    HttpClient.prototype.postMultipart = function (requestData, urlPath, callback) {
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(requestData, "requestData");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(requestData.agentData, "requestData.agentData");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(requestData.reportFile, "requestData.reportFile");
        var agentData = requestData.agentData;
        var reportFile = requestData.reportFile;
        try {
            var opts = this.createDefaultOptions(urlPath);
            var id = uuid();
            var onRequestCallback = this.createOnRequestCallback(http_verb_1.HttpVerb.POST, id, callback);
            this.logger.info("Sending " + http_verb_1.HttpVerb.POST + " request. Url: '" + opts.url + "', requestId: '" + id + "'");
            var req = request.post(opts, onRequestCallback);
            var form = req.form();
            form.append('file', JSON.stringify(agentData), {
                filename: 'agentData',
                contentType: 'multipart/form-data'
            });
            form.append('file', reportFile.buffer.toString(), {
                filename: 'report',
                contentType: 'multipart/form-data'
            });
        }
        catch (err) {
            this.logger.error("Failed sending Http " + http_verb_1.HttpVerb.POST + " request to:'" + urlPath + "'. Error: '" + err + "'");
            return callback(err, null);
        }
    };
    HttpClient.prototype.invokeHttpRequest = function (httpVerb, urlPath, callback, additionalHeaders, buffer, contentType) {
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(httpVerb, "httpVerb");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(urlPath, "urlPath");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(callback, "callback");
        try {
            var opts = this.createDefaultOptions(urlPath, contentType);
            if (additionalHeaders) {
                for (var header in additionalHeaders) { //Copy additional headers
                    opts.headers[header] = additionalHeaders[header];
                }
            }
            var id = uuid();
            var onRequestCallback = this.createOnRequestCallback(httpVerb, id, callback);
            this.logger.info("Sending " + httpVerb + " request. Url: '" + opts.url + "', requestId: '" + id + "'");
            if (httpVerb === http_verb_1.HttpVerb.GET) {
                opts.json = true;
                request.get(opts, onRequestCallback);
            }
            else if (httpVerb === http_verb_1.HttpVerb.DELETE) {
                opts.json = true;
                request.delete(opts, onRequestCallback);
            }
            else if (httpVerb === http_verb_1.HttpVerb.POST) {
                opts.body = buffer;
                request.post(opts, onRequestCallback);
            }
            else {
                new Error(httpVerb + " is not implemented yet.");
            }
        }
        catch (err) {
            this.logger.error("Failed sending Http " + httpVerb + " request to:'" + urlPath + "'. Error: '" + err + "'");
            return callback(err, null);
        }
    };
    HttpClient.prototype.createOnRequestCallback = function (httpVerb, requestId, callback) {
        var _this = this;
        var handler = function (err, requestResponse, body) {
            if (err || ((requestResponse != null) && (requestResponse.statusCode < 200 || requestResponse.statusCode >= 400))) {
                var statusCode = -1;
                if (requestResponse != null) {
                    statusCode = requestResponse.statusCode;
                    _this.logger.warn("Got " + statusCode + " in '" + httpVerb + "' request. Request id:'" + requestId + "'.");
                }
                if (err) {
                    err = new Error("HttpClient failed. Error: " + err.message + ", StatusCode: " + statusCode + ", Stack: " + err.stack);
                }
                else if (statusCode === 404) {
                    //404 is valid since in REST we have situations in which 404 represents an empty collection
                    err = null;
                }
                else {
                    err = new Error("Server returned " + statusCode + " status code in '" + httpVerb + "' request. Request id:'" + requestId + "'.");
                }
                return callback(err, null);
            }
            if (body && body.length > 0) {
                body = JSON.parse(body);
            }
            _this.logger.info("'" + httpVerb + "' request was completed successfully. Request id:'" + requestId + "'. statusCode: " + requestResponse.statusCode + " body: " + body);
            return callback(null, body);
        };
        return handler;
    };
    HttpClient.prototype.allowUntrustedCertificates = function () {
        //For more information: https://stackoverflow.com/questions/17383351/how-to-capture-http-messages-from-request-node-library-with-fiddler/22996076#22996076
        process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';
    };
    HttpClient.prototype.createDefaultOptions = function (urlPath, contentType) {
        if (contentType === void 0) { contentType = DEFAULT_CONTENT_TYPE; }
        //get
        var opts = {
            url: this.cfg.server + urlPath,
            headers: {
                "Content-Type": contentType,
                "Authorization": "Bearer " + this.cfg.token
            },
            timeout: this.defaultTimeout
        };
        //TODO: Verify if we need it. Nadav, 21.12.18
        opts["compressed"] = true;
        if (this.cfg.proxy != null) { //http://user:pass@proxy.server.com:3128
            opts.proxy = this.cfg.proxy;
            this.allowUntrustedCertificates();
        }
        return opts;
    };
    HttpClient.prototype.getDefaultTimeout = function () {
        var timeout = sl_env_vars_1.SlEnvVars.getHttpTimeout();
        if (timeout == null) {
            timeout = 60 * 1000 * 2;
        }
        timeout = Number(timeout);
        this.logger.debug("Http timeout value is '" + timeout + "'.");
        return timeout;
    };
    return HttpClient;
}());
exports.HttpClient = HttpClient;
//# sourceMappingURL=http-client.js.map