"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sl_env_vars_1 = require("./constants/sl-env-vars");
var fs = require('fs');
var parser = require("gitignore-parser");
var files_utils_1 = require("./utils/files-utils");
var IgnoredFilesHandler = /** @class */ (function () {
    function IgnoredFilesHandler(slIgnoreFilePath, outputPath, filename, logger) {
        this.slIgnoreFilePath = slIgnoreFilePath;
        this.outputPath = outputPath;
        this.filename = filename;
        this.logger = logger;
        this.parsed = this.getParser();
    }
    IgnoredFilesHandler.prototype.getParser = function () {
        var pattern = this.getIgnorePattern();
        try {
            var compiled = parser.compile(pattern); //may throw if the file is invalid/corrupted
            return compiled;
        }
        catch (e) {
            this.logger.error("Error while trying to use .slignore pattern, pattern seems to be corrupted. Using default behavior.");
            pattern = this.getDefaultPattern();
            return parser.compile(pattern); //should not throw again
        }
    };
    IgnoredFilesHandler.prototype.getIgnorePattern = function () {
        if (!this.slIgnoreFilePath) {
            this.logger.info("'%s' file does not exists, trying to load nyc configuration.", this.filename);
            var patternFromNyc = this.readFromNycConfig();
            if (patternFromNyc) {
                return patternFromNyc;
            }
            this.logger.info("Could not find 'exclude' in nyc configuration, using default behavior");
            return this.getDefaultPattern();
        }
        try {
            var pattern = fs.readFileSync(this.slIgnoreFilePath, 'utf8');
            this.logger.info("Using '%s' file '%s'", this.filename, this.slIgnoreFilePath);
            return this.addExcludedPath(pattern, this.outputPath);
        }
        catch (err) {
            this.logger.info("Failed to read from: '%s'. Using default behavior.", this.slIgnoreFilePath);
            return this.getDefaultPattern();
        }
    };
    IgnoredFilesHandler.prototype.addExcludedPath = function (pattern, excludedFolder) {
        if (!excludedFolder)
            return pattern;
        if (excludedFolder && excludedFolder[excludedFolder.length - 1] != '/') { //Ensure the excludedFolder ends with / to comply with gitignore
            excludedFolder += '/';
        }
        pattern = pattern + "\n" + excludedFolder;
        return pattern;
    };
    IgnoredFilesHandler.prototype.getDefaultPattern = function () {
        // default pattern - all js files under test directory and it's sub directories.
        var pattern = "";
        pattern = this.addExcludedPath(pattern, this.outputPath);
        return pattern;
    };
    IgnoredFilesHandler.prototype.readFromNycConfig = function () {
        try {
            var cwd = sl_env_vars_1.SlEnvVars.getBasePath() || process.cwd();
            var excludeArr = void 0;
            var rcFilePath = files_utils_1.FilesUtils.findFileUp(".nycrc", cwd) || files_utils_1.FilesUtils.findFileUp(".nycrc.json", cwd);
            ;
            var packageJsonPath = files_utils_1.FilesUtils.findFileUp("package.json", cwd);
            if (rcFilePath) {
                this.logger.debug("Reading nyc exclude from '%s'", rcFilePath);
                excludeArr = this.readNycConfig(rcFilePath, false);
            }
            else if (packageJsonPath) {
                this.logger.debug("Reading nyc exclude from '%s'", packageJsonPath);
                excludeArr = this.readNycConfig(packageJsonPath, true);
            }
            return this.createExcludePatternFromNyc(excludeArr);
        }
        catch (err) {
            this.logger.warn("Error while trying to read nyc configuration. Error: ", err);
            return null;
        }
    };
    IgnoredFilesHandler.prototype.readNycConfig = function (file, isPackageJson) {
        var fileContent = fs.readFileSync(file);
        var configObject = JSON.parse(fileContent);
        var nycConfig = isPackageJson ? configObject.nyc : configObject;
        if (nycConfig) {
            return nycConfig.exclude;
        }
        return null;
    };
    IgnoredFilesHandler.prototype.createExcludePatternFromNyc = function (excludeArr) {
        if (!excludeArr) {
            this.logger.info("No files to exclude in nyc configuration");
            return null;
        }
        this.logger.debug("Found the following exclude patterns: " + JSON.stringify(excludeArr, null, "\t"));
        return excludeArr.join("\n");
    };
    IgnoredFilesHandler.prototype.denies = function (input) {
        return this.parsed.denies(input);
    };
    IgnoredFilesHandler.prototype.accepts = function (input) {
        return this.parsed.accepts(input);
    };
    IgnoredFilesHandler.SCAN_IGNORE_FILE_NAME = ".slignore";
    IgnoredFilesHandler.GENERATED_IGNORE_FILE_NAME = ".slignore.generated";
    return IgnoredFilesHandler;
}());
exports.IgnoredFilesHandler = IgnoredFilesHandler;
//# sourceMappingURL=ignored-files-handler.js.map