"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var validation_utils_1 = require("./utils/validation-utils");
var path = require("path");
var files_utils_1 = require("./utils/files-utils");
/**
 * Resolve the source data for method or branch.
 * Resolves the followingg:
 * Relative path of source file.
 * Absolute path of source file.
 * Start position in source file.
 * End position in source file.
 * The name of original methid (in case of method).
 */
var SourceMapHandler = /** @class */ (function () {
    function SourceMapHandler(consumer, generatedFileName, generatedStart, generatedEnd, projectRoot, logger) {
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(consumer, "consumer");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(generatedFileName, "generatedFileName");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(generatedStart, "generatedStart");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(generatedEnd, "generatedEnd");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(logger, "logger");
        this.consumer = consumer;
        this.generatedFileName = generatedFileName;
        this.projectRoot = projectRoot;
        this.logger = logger;
        this._start = this.resolveOriginalPosition(generatedStart, false);
        this._end = this.resolveOriginalPosition(generatedEnd, true);
    }
    SourceMapHandler.prototype.resolvePaths = function () {
        if (!this.isValidPositionObject(this._start)) {
            this.logger.debug("Invalid original start position '" + this._start + "', skipping this method");
            this._shouldSkipped = true;
            return;
        }
        this.originalName = this._start.name;
        var source = this.trimWebpackPrefix(this._start.source);
        if (this.isThirdPartyCode(source)) {
            this.logger.debug("File '" + source + "' is a third party file, marked as skipped");
            this._shouldSkipped = true;
            return;
        }
        if (process.env.SL_IS_REACT) {
            source = source.replace(SourceMapHandler.REACT_GENERATED_FOLDER, SourceMapHandler.REACT_SOURCE_FOLDER);
        }
        this.resolveAbsolutePath(source);
        if (this._shouldSkipped) {
            return;
        }
        this.resolveRelativePath();
    };
    ;
    SourceMapHandler.prototype.setScmParams = function (scmFilesContainer, scmRootDir) {
        this.scmFilesContainer = scmFilesContainer;
        this.scmRootDir = scmRootDir;
    };
    Object.defineProperty(SourceMapHandler.prototype, "absolutePath", {
        get: function () {
            return this._absolutePath;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SourceMapHandler.prototype, "relativePath", {
        get: function () {
            return this._relativePath;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SourceMapHandler.prototype, "start", {
        get: function () {
            return this._start;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SourceMapHandler.prototype, "end", {
        get: function () {
            return this._end;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SourceMapHandler.prototype, "shouldSkipped", {
        get: function () {
            return this._shouldSkipped;
        },
        enumerable: true,
        configurable: true
    });
    SourceMapHandler.prototype.resolveAbsolutePath = function (source) {
        source = files_utils_1.FilesUtils.adjustPathSlashes(source);
        var absolutePath;
        if (path.isAbsolute(source)) {
            this.logger.debug("file '" + source + "' is absolute");
            absolutePath = source;
        }
        else if (this.projectRoot) {
            if (source.indexOf(this.projectRoot) === 0) {
                this.logger.debug("Given projectRoot '" + this.projectRoot + "' is already at the beginning of '" + source + "'");
                absolutePath = source;
            }
            else {
                absolutePath = path.join(this.projectRoot, source);
            }
        }
        else if (this.scmFilesContainer) {
            absolutePath = this.resolvePathsByScm(source);
        }
        else {
            var dirname = path.dirname(this.generatedFileName);
            absolutePath = path.resolve(dirname, source);
        }
        this._absolutePath = files_utils_1.FilesUtils.adjustPathSlashes(absolutePath);
        this.logger.debug("Absolute path for '" + source + "' resolved to be '" + this._absolutePath + "'");
    };
    SourceMapHandler.prototype.resolveRelativePath = function () {
        var prefixToTrim = this.projectRoot || this.scmRootDir || process.cwd();
        prefixToTrim = files_utils_1.FilesUtils.adjustPathSlashes(prefixToTrim);
        if (this._absolutePath.indexOf(prefixToTrim) === 0) {
            this._relativePath = this._absolutePath.replace(prefixToTrim, "");
            if (this._relativePath.indexOf("/") === 0) {
                this._relativePath = this._relativePath.substring(1);
            }
        }
        else {
            this.logger.warn("Prefix '" + this.projectRoot + "' is not in the start of '" + this._absolutePath + "', could not resolve relative path");
            this._relativePath = this._absolutePath;
        }
    };
    SourceMapHandler.prototype.resolvePathsByScm = function (source) {
        var gitFiles = this.scmFilesContainer.getScmPath(source);
        if (!gitFiles) {
            this.logger.warn("File '" + source + "' not found in git files, marked as skipped");
            this._shouldSkipped = true;
            return null;
        }
        else if (gitFiles.length > 1) {
            // TODO: report to onboarding dashboard
            this.logger.warn("File '" + source + "' match to multiple git files, absolute path resolved relative ");
            return path.join(this.scmRootDir, source);
        }
        else {
            return path.join(this.scmRootDir, gitFiles[0]);
        }
    };
    SourceMapHandler.prototype.resolveOriginalPosition = function (generatedPosition, isEnd) {
        var originalPosition = this.consumer.originalPositionFor({
            line: generatedPosition[0],
            column: isEnd ? generatedPosition[1] - 1 : generatedPosition[1]
        });
        return new Position(originalPosition);
    };
    ;
    SourceMapHandler.prototype.isValidPositionObject = function (position) {
        return position && position.source && position.line !== null && position.column !== null;
    };
    SourceMapHandler.prototype.trimWebpackPrefix = function (source) {
        if (source.indexOf(SourceMapHandler.WEBPACK_PREFIX) === 0) {
            source = source.substring(SourceMapHandler.WEBPACK_PREFIX.length);
        }
        return source;
    };
    SourceMapHandler.prototype.isThirdPartyCode = function (source) {
        return SourceMapHandler.THIRD_PARTIES_FOLDERS.some(function (folder) { return source.indexOf(folder) > -1; });
    };
    SourceMapHandler.WEBPACK_PREFIX = "webpack:///";
    SourceMapHandler.REACT_GENERATED_FOLDER = "build/static/js";
    SourceMapHandler.REACT_SOURCE_FOLDER = "src";
    SourceMapHandler.WEBPACK_BOOTSTRAP_FILE = "webpack/bootstrap";
    SourceMapHandler.WEBPACK_LAZY_FILE = "$_lazy";
    SourceMapHandler.WEBPACK_SRC_ASYNC = "src async";
    SourceMapHandler.WEBPACK_SRC_LAZY = "src lazy";
    SourceMapHandler.WEBPACK_RESOURCE_SIGN = "~"; // Webpack replace 'node_modules' with '~'
    SourceMapHandler.NODE_MODULES = "node_modules";
    SourceMapHandler.THIRD_PARTIES_FOLDERS = [
        SourceMapHandler.WEBPACK_BOOTSTRAP_FILE,
        SourceMapHandler.WEBPACK_LAZY_FILE,
        SourceMapHandler.WEBPACK_RESOURCE_SIGN,
        SourceMapHandler.NODE_MODULES,
        SourceMapHandler.WEBPACK_SRC_ASYNC,
        SourceMapHandler.WEBPACK_SRC_LAZY
    ];
    return SourceMapHandler;
}());
exports.SourceMapHandler = SourceMapHandler;
var Position = /** @class */ (function () {
    function Position(position) {
        this.column = position.column;
        this.line = position.line;
        this.name = position.name;
        this.source = position.source;
    }
    Position.prototype.asFormatArray = function () {
        return [this.line, this.column];
    };
    return Position;
}());
//# sourceMappingURL=source-map-handler.js.map