"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ConvertSourceMap = require("convert-source-map");
var path = require("path");
var fs = require("fs");
var validation_utils_1 = require("./utils/validation-utils");
/**
 * Reads source map data from generated file, uses 'convert-source-map' packge to resolve.
 * If not found trying to search relevat map file under same directory as gem=nerated.
 */
var SourceMapReader = /** @class */ (function () {
    function SourceMapReader(filename, logger) {
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(filename, "filename");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(logger, "logger");
        this.filename = filename;
        this.logger = logger;
    }
    SourceMapReader.prototype.readSourceMapData = function (generatedContent) {
        if (!generatedContent) {
            this.logger.debug("'generatedContent' cannot be null or empty");
            return null;
        }
        this.logger.debug("About to resolve source map data for '" + this.filename + "'");
        var sourceMapData = ConvertSourceMap.fromSource(generatedContent) ||
            ConvertSourceMap.fromMapFileSource(generatedContent, path.dirname(this.filename));
        if (sourceMapData && sourceMapData.sourcemap) {
            this.logger.info("Found source map data for '" + this.filename + "'");
            return sourceMapData.sourcemap;
        }
        else {
            var mapFromFile = this.readFromFile();
            if (mapFromFile) {
                return mapFromFile;
            }
            else {
                if (this.isUsingSourceURL(generatedContent)) {
                    //TODO: Fix message
                    this.logger.info("Seems source map comment is in unsupported format...");
                }
                this.logger.debug("Couldn't find source map data for '" + this.filename + "'");
                return null;
            }
        }
    };
    SourceMapReader.prototype.readFromFile = function () {
        var sourceMap = null;
        var mapFileName = this.filename + SourceMapReader.MAP_FILE_EXTENSION;
        var isMapFileExists = fs.existsSync(mapFileName);
        if (!isMapFileExists) {
            this.logger.info("File '" + mapFileName + "' not exists.");
            return null;
        }
        this.logger.info("Found map file '" + mapFileName + "'");
        try {
            var rawMapData = fs.readFileSync(mapFileName).toString();
            sourceMap = JSON.parse(rawMapData);
            return sourceMap;
        }
        catch (e) {
            this.logger.error("Could not parse source map data from '" + path);
            this.logger.error("Error: '" + e);
            return null;
        }
    };
    /**
     *  The 'convert-source-map' detects only comments using 'sourceMappingURL' which is the new standard,
     some tools generate with comment 'sourceURL' which not supported
     */
    SourceMapReader.prototype.isUsingSourceURL = function (content) {
        var fileMatch = content.match(SourceMapReader.MAP_FILE_COMMENT_REGEX);
        var commentMatch = content.match(SourceMapReader.COMMENT_REGEX);
        return commentMatch !== null || fileMatch !== null;
    };
    /*
        The following regex's based on regex used in 'convert-source-map' to detect source map.
        See https://github.com/thlorenz/convert-source-map/blob/master/index.js
     */
    SourceMapReader.COMMENT_REGEX = /^\s*\/(?:\/|\*)[@#]\s+sourceURL=data:(?:application|text)\/json;(?:charset[:=]\S+?;)?base64,(?:.*)$/mg;
    SourceMapReader.MAP_FILE_COMMENT_REGEX = /(?:\/\/[@#][ \t]+sourceURL=([^\s'"`]+?)[ \t]*$)|(?:\/\*[@#][ \t]+sourceURL=([^\*]+?)[ \t]*(?:\*\/){1}[ \t]*$)/mg;
    SourceMapReader.MAP_FILE_EXTENSION = ".map";
    return SourceMapReader;
}());
exports.SourceMapReader = SourceMapReader;
//# sourceMappingURL=source-map-reader.js.map